"""
                       888
                       888
                       888
88888b.  888  888  .d88888  .d88b.
888 "88b 888  888 d88" 888 d88""88b
888  888 888  888 888  888 888  888
888 d88P Y88b 888 Y88b 888 Y88..88P
88888P"   "Y88888  "Y88888  "Y88P"
888           888
888      Y8b d88P
888       "Y88P"

A simple terminal TODO app written in Python 3.

Copyright (C) 2018 Connor Ruggles

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import argparse
import os
import sys
import threading
from . import config
from . import utils
from . import interactions
from . import conf_parser

LIST_FILE = os.environ["HOME"] + "/.local/pydo/todo.json"
CONF_FILE = os.environ["HOME"] + "/.local/pydo/pydo.ini"
FILE_LOCK = threading.RLock()

def startup(args):
    """just prints a welcome message and gets arguments"""
    arguments = argparse.ArgumentParser(description="A simple terminal TODO app.")
    arguments.add_argument("-f", metavar="\"path/to/file\"", \
            help="Path to a list file named/placed differently than the default.")
    arguments.add_argument("-c", metavar="\"path/to/file\"", \
            help="Path to a conf file named/placed differently than the default.")
    arguments.add_argument("-v", action="store_true", \
            help="Prints version number of pydo.")
    return arguments.parse_args(args)

def process_args(args):
    """processes arguments"""
    list_dest = LIST_FILE
    conf_dest = CONF_FILE
    if args.f:
        print("An alternate list file was given: ", args.f)
        list_dest = args.f
    if args.c:
        print("An alternate conf file was given: ", args.c)
        conf_dest = args.c
    if args.v:
        print("pydo " + config.__version__)
        sys.exit(0)
    conf_parser.parse_config(conf_dest)
    return utils.setup_list(list_dest, FILE_LOCK)

def main():
    """main"""
    arguments = startup(sys.argv[1:])
    task_list = process_args(arguments)
    utils.clear()
    print("\n+-----------------+")
    print("| Welcome to pydo |")
    print("+-----------------+\n")
    print("PyDo  Copyright (C) 2018  Connor Ruggles\n"
          "This program comes with ABSOLUTELY NO WARRANTY; for details type `show w'.\n"
          "This is free software, and you are welcome to redistribute it\n"
          "under certain conditions; refer to the License for details.\n")
    while True:
        usr_input = interactions.get_intent()
        if usr_input == "l" or usr_input == "L":
            interactions.print_all_tasks(task_list)
        elif usr_input == "c" or usr_input == "C":
            interactions.create_task(task_list)
        elif usr_input == "d" or usr_input == "D":
            interactions.delete_task(task_list)
        elif usr_input == "f" or usr_input == "F":
            interactions.finish_task(task_list)
        elif usr_input == "q" or usr_input == "Q":
            print("\nWaiting for sync to finish...")
            destroy_success = task_list.destroy()
            if destroy_success == 0:
                print("Success")
            elif destroy_success is None:
                pass
            else:
                print("There was a problem syncing the file.")
            print("Goodbye.")
            sys.exit(0)
        elif usr_input == "show w":
            interactions.show_w()
        else:
            print("Invalid option.")

main()



"""
A class that holds a TaskList class and related utilities.

Copyright (C) 2018 Connor Ruggles

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import json
from . import file_utils
from . import Task

class TaskList(list):
    """holds a list of Tasks"""

    def __init__(self, list_file, file_lock):
        """initializes this list based on the passed in file"""
        list.__init__(self)
        self._list_file = list_file
        self._file_lock = file_lock
        self._task_list = []
        self._file_lock.acquire()
        file_utils.get_remote(self._list_file)
        self.retrieve_tasks_from_file()
        self._file_lock.release()

    def __getitem__(self, index):
        """implements the getitem method to iterate over task list"""
        result = self._task_list[index]
        return result

    def __len__(self):
        """len method"""
        return len(self._task_list)

    def destroy(self):
        """performs final tasks before exiting"""
        p = file_utils.send_remote(self._list_file)
        return p

    def retrieve_tasks_from_file(self):
        """function name says it all"""
        self._task_list = []
        with open(self._list_file) as json_file:
            data = json.load(json_file)
            for tmp_task in data:
                self._task_list.append(Task.Task(tmp_task["title"], \
                    tmp_task["description"], tmp_task["done"], tmp_task["id"]))

    def add_task(self, task_to_add):
        """adds a task to this list"""
        self._task_list.append(task_to_add)
        self.reload_tasks()

    def delete_task(self, index):
        """deletes a task"""
        del self._task_list[index]
        self.reload_tasks()

    def finish_task(self, index):
        """finishes a task"""
        self._task_list[index].finish()
        self.reload_tasks()

    def get_tasks(self):
        """retrieves all tasks in this task list"""
        return self._task_list

    def print_tasks(self):
        """prints the list of tasks"""
        for _task in self._task_list:
            print("Title      : {}".format(_task.title))
            if _task.description != "":
                print("Description: {}".format(_task.description))
            print("Done       : {}\n".format(str(_task.done)))

    def reload_tasks(self):
        """
        gets the task list from the file and reloads the
        task list
        """
        tmp_list = []
        # first iterate through the task list,
        # appending a dict form of each task to a
        # temporary list
        for task_obj in self._task_list:
            tmp_list.append(task_obj.__dict__)
        self._file_lock.acquire()
        with open(self._list_file, 'w') as outfile:
            # then dump that temp list to the file
            json.dump(tmp_list, outfile)
            outfile.close()
        # reset the task list
        self._task_list = []
        # re-open the file, and add each entry as a
        # new Task
        self.retrieve_tasks_from_file()
        self._file_lock.release()



"""
contains all the utilities needed for remote file work

Copyright (C) 2018 Connor Ruggles

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import subprocess
from .conf_parser import CONFIG

def send_remote(list_file):
    """syncs the remote task file with this one"""
    if "url" in CONFIG:
        with open("/dev/null", "w") as devnull:
            p = subprocess.call(["scp", list_file, CONFIG["url"]["dest"]], stdout=devnull)
            devnull.close()
        return p
    else:
        return None

def get_remote(list_file):
    """gets the remote task file"""
    if "url" in CONFIG:
        with open("/dev/null", "w") as devnull:
            p = subprocess.call(["scp", CONFIG["url"]["dest"], list_file], stdout=devnull)
            devnull.close()
        return p
    else:
        return None



"""
A class that holds data about an individual task.

Copyright (C) 2018 Connor Ruggles

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

class Task:
    """
    A class that holds data about an individual task.
    """
    def __init__(self, title, description, done=False, task_id=None):
        self.id = task_id
        self.title = title
        self.description = description
        self.done = done
        self.__dict__ = {
            "id": self.id,
            "title": self.title,
            "description": self.description,
            "done": self.done,
        }

    def finish(self):
        """sets this task to done"""
        self.done = True



"""
                       888
                       888
                       888
88888b.  888  888  .d88888  .d88b.
888 "88b 888  888 d88" 888 d88""88b
888  888 888  888 888  888 888  888
888 d88P Y88b 888 Y88b 888 Y88..88P
88888P"   "Y88888  "Y88888  "Y88P"
888           888
888      Y8b d88P
888       "Y88P"

A simple terminal TODO app written in Python 3.
"""

from . import List
from . import Task
from . import config
from . import utils
from . import interactions
from . import conf_parser
from . import file_utils



"""
Copyright (C) 2018 Connor Ruggles

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import os
import sys
from .List import TaskList

def clear():
    """clears the screen"""
    _ = os.system('clear')

def setup_list(list_dest, file_lock):
    """sets up the todo list, which should be a json file"""
    check_file(list_dest)
    return TaskList(list_dest, file_lock)

def check_file(list_file):
    """checks for the existence of the todo list file"""
    list_file = os.path.expanduser(list_file)
    if os.path.exists(list_file):
        if not os.path.isfile(list_file):
            print("Arguments with '-f' need to be a file.")
            sys.exit(1)
        if os.path.splitext(list_file)[1] != ".json":
            print("File given must be a .json file.")
            sys.exit(1)
    else:
        os.makedirs(os.path.abspath(os.path.dirname(list_file)), exist_ok=True)
        with open(list_file, 'w+') as new_list:
            new_list.close()



"""
Copyright (C) 2018 Connor Ruggles

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

from . import Task
from . import utils

def get_intent():
    """gets the intent of the user"""
    print("What would you like to do?\n")
    return input("(l)ist tasks; (c)reate task; (d)elete task; (f)inish task; (q)uit\n> ")

def print_all_tasks(list_object):
    """
    calls a function to retrieve all
    tasks, then prints them
    """
    utils.clear()
    print("All tasks:")
    print("----------")
    list_object.print_tasks()

def create_task(list_object):
    """
    creates a task with user input of
    title and description
    """
    utils.clear()
    print("New task")
    print("--------")
    title = input("Title of new task: ")
    desc = input("Description of new task (leave blank for no description): ")
    new_task = Task.Task(title, desc)
    list_object.add_task(new_task)
    print("New Task successfully created.\n")

def delete_task(list_object):
    """deletes the selected task"""
    utils.clear()
    print("Deleting task")
    print("-------------")
    list_object.print_tasks()
    index = input("Which task to delete? (1 for first task, 2 for second, etc.)\n> ")
    list_object.delete_task(int(index) - 1)

def finish_task(list_object):
    """finishes the selected task"""
    utils.clear()
    print("Finishing task")
    print("--------------")
    list_object.print_tasks()
    index = input("Which task to finish? (1 for first task, 2 for second, etc.)\n> ")
    list_object.finish_task(int(index) - 1)

def show_w():
    """prints the warranty section of GPL"""
    utils.clear()
    print("THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY\n"
          "APPLICABLE LAW.  EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT\n"
          "HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM \"AS IS\" WITHOUT WARRANTY\n"
          "OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,\n"
          "THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR\n"
          "PURPOSE.  THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM\n"
          "IS WITH YOU.  SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF\n"
          "ALL NECESSARY SERVICING, REPAIR OR CORRECTION.\n")



"""
setup.py

Copyright (C) 2018 Connor Ruggles

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import setuptools

try:
    import pydo
except ImportError:
    print("pydo requires python 3.5 or greater. Exiting.")
    quit(1)

setuptools.setup(
    name='pydolist',
    version=pydo.config.__version__,
    author=pydo.config.AUTHOR,
    author_email=pydo.config.EMAIL,
    description='A simple terminal TODO app written in Python 3.',
    license="GPLv3",
    url=pydo.config.URL,
    packages=["pydo"],
    entry_points={"console_scripts": ["pydo=pydo.__main__:main"]},
    python_requires=">=3.5",
    zip_safe=False
)



"""
parses the pydo.conf file

Copyright (C) 2018 Connor Ruggles

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import configparser
import os
import sys

CONFIG = configparser.ConfigParser()

def parse_config(conf_dest):
    """parses the config file for pydo at the passed in location"""
    conf_dest = os.path.expanduser(conf_dest)
    if os.path.exists(conf_dest):
        if not os.path.isfile(conf_dest):
            print("pydo: Conf file needs to be a file.")
            sys.exit(1)
        CONFIG.read(conf_dest)
    else:
        print("pydo: File " + conf_dest + " doesn't exist. Not using config file.")
